'NV_Ver1_2.bas
'Holiday Lights Controller
'Peter J. Stonard
'
'Released August1, 08
'
'  ####################################################################
'  # Final ver 1.2 with thirty second pattern step                    #
'  # Bug Fixes:                                                       #
'  # Bug: Welcome Screen changed from 1.1 to 1.2                      #
'  # Bug: Show timer wouldn't run past midnight.                      #
'  # Bug: LCD would get confused                                      #
'  # Bug: Data on the LEDs would be scrambled                         #
'  # Bug: End of the lightshow, turn off all lamps                    #
'  # Bug: If lightshow turned on manually, last pattern sticks        #
'  #                                                                  #
'  ####################################################################

'****************************************************************************
'* Hardware Definition                                                      *
'****************************************************************************
'I2C bus connects a DS1307, an EEPROM, I2C output to other cards, LCD 16x1
'and a MUX two digit LED display
'Bit7 = MSB
'
'Encoder, Switch, 120Hz Int0, LCD data on Port D.
'PD7 Does Not Exist! Always Set to zero
'
'PD6 = DB0
'PD5 = DB1
'PD4 = DB2
'PD3 = DB3
'PD2 = 120Hz
'PD1 = ENCA
'PD0 = ENCB
'
'LCD on Port C.
'PC7 = Does Not Exist! Always Set to zero
'PC6 = RESET*
'PC5 = DB7
'PC4 = DB6
'PC3 = DB5
'PC2 = DB4
'PC1 = E
'PC0 = RS
'
'Prog, I2C, LED Anodes on Port B.
'PB7 = SEL_R*
'PB6 = SEL_L*
'PB5 = SCK
'PB4 = MISO
'PB3 = MOSI
'PB2 = not used
'PB1 = SCL  (I2C Data)
'PB0 = SDA  (I2C Clock)
'
'****************************************************************************
'* Define Variables                                                         *
'****************************************************************************
'Variables stored as a bit
Dim Enc_dir As Bit
Dim Int_1sec As Bit
Dim Int_3sec As Bit
Dim Int_esec As Bit
Dim Int_hsec As Bit
Dim Int_qsec As Bit
Dim Int_1sec1 As Bit
Dim Int_1sec2 As Bit
Dim Int_3sec1 As Bit
Dim Int_esec1 As Bit
Dim Int_lcd As Bit
Dim Int_hsec1 As Bit
Dim Int_qsec1 As Bit

Dim Led_en As Bit
Dim Lcd_en As Bit
Dim Led_inv As Bit
Dim Mode_data As Bit
Dim New_sw As Bit
Dim Old_sw As Bit
'
Dim R_eeprom As Bit
Dim R_rtc As Bit
Dim R_stat As Bit
Dim R_stop As Bit
Dim R_strt As Bit
'
Dim Read_sw As Bit
Dim Ro_flg As Bit
Dim Rtc_en As Bit
Dim Show As Bit
Dim Swing As Bit
Dim Up_dwn As Bit
'
Dim W_eeprom As Bit
Dim W_rtc As Bit
Dim W_stat As Bit
Dim W_stop As Bit
Dim W_strt As Bit
'
' Variables stored as a byte
Dim 1sec_cnt As Byte                    'counter for one second interrupt
Dim 3sec_cnt As Byte                    'counter for three second interrupt
Dim Adresh As Byte                      'EEPROM device read address
Dim Adresl As Byte                      'EEPROM device write address
Dim Ch_data As Byte                     'Number of IO channels available
Dim Cnt1 As Byte                        'Count timer cycles (64 = 1/2 AC cycle)
Dim Cnt2 As Byte                        'Count phase delay
Dim Cnt_lcd As Byte                     'LCD cursor counter ( 1 to 16)
Dim Data_hr As Byte                     'Time hour storage
Dim Data_min As Byte                    'Time min storage
Dim Data_sec As Byte                    'Time sec storage
Dim Display_mode As Byte                'LCD Display mode
Dim Done As Byte                        'Fader flag
Dim Ee_dat1 As Byte                     'EEPROM data lower nibble as byte
Dim Ee_dat2 As Byte                     'EEPROM data upper nibble as byte
'
Dim Enc_all As Byte                     'Encoder memory
'Dim Enc_data As Byte                    'Encoder data (0 to 100)
Dim Enc_last As Byte                    'Encoder memory
Dim Enc_now As Byte                     'Encoder memory

Dim Enc_mode As Byte                    'Format of encoder data
Dim Esec_cnt As Byte                    'counter for eighth second interrupt
Dim Hsec_cnt As Byte                    'counter for half second interrupt
Dim On_data(32) As Byte                 'Output data array
Dim Pat_dly As Byte                     'Pattern delay counter
Dim Pat_end As Byte                     'Time in seconds of current pattern
Dim Pat_num As Byte                     'Current pattern
Dim Pos_data As Byte                    'Temp Position data
Dim Qsec_cnt As Byte                    'counter for half second interrupt
Dim Ramp As Byte                        'Wave effect data
Dim Rtc_hr As Byte                      'RTC hour storage
Dim Rtc_min As Byte                     'RTC hour storage
Dim Rtc_sec As Byte                     'RTC hour storage
Dim Seed1 As Byte                       'Timer1 counter value
Dim Stop_hr As Byte                     'Stop hour storage BCD
Dim Stop_min As Byte                    'Stop min storage BCD
Dim Data_hr_d As Byte                   'Stop hour storage Decimal
Dim Data_min_d As Byte                  'Stop min storage Decimal
Dim Status As Byte                      'Status storage
Dim Strt_hr As Byte                     'Start hour storage
Dim Strt_min As Byte                    'Start min storage
Dim Switch_data As Byte
'
Dim Tmpb_a As Byte                      'Temp storage
Dim Tmpb_b As Byte                      'Temp storage
Dim Tmpb_c As Byte                      'Temp storage
'
Dim Tempi As Byte                       'Temp storage
Dim Tempm As Byte                       'Temp storage
Dim Tempn As Byte                       'Temp storage
Dim Tmp_ee1 As Byte                     'Temp storage
Dim Tmp_ee2 As Byte                     'Temp storage
Dim Lamp_dat1(8) As Byte                'Output data array
Dim Lamp_dat2(8) As Byte                'Output data array
'Dim Lamp_dat3(8) As Byte                'Output data array
'Dim Lamp_dat4(8) As Byte                'Output data array
'
' Variables stored as a word
Dim Led_data As Word
Dim Led_data1 As Word
Dim Led_data2 As Word
Dim Led_data3 As Word
Dim Now_mins As Word                    'Current time in minutes
Dim St_mins As Word                     'Start time in minutes
Dim Sp_mins As Word                     'Stop time in minutes
Dim Tmpw_a As Word                      'Temp storage
Dim Tempw As Word                       'Temp storage
Dim Tempx As Word                       'Temp storage
Dim Tempy As Word                       'Temp storage
Dim Char_d(23) As Word                  'Normal Char look up table
Dim Char_u(23) As Word                  'Inverted Char look up table
'
' Variables stored as strings
Dim Lcd_msg As String * 16              'Current message storage space
'****************************************************************************
'* Alias                                                                    *
'****************************************************************************
Switch Alias Pind.3
Enca Alias Pind.1
Encb Alias Pind.0
'****************************************************************************
'* Define Subroutines                                                       *
'****************************************************************************
Declare Sub Clear_arrays
Declare Sub Read_eeprom
Declare Sub Read_stat
Declare Sub Read_stop
Declare Sub Read_strt
Declare Sub Start_rtc
Declare Sub Write_eeprom
Declare Sub Write_stat
Declare Sub Write_stop
Declare Sub Write_strt
'****************************************************************************
'* Define AVR                                                               *
'****************************************************************************
$regfile = "m168def.dat"                'Change for your AVR.
$crystal = 8000000                      '8 MHz Crystal.
Config Lcdpin = Pin , Db4 = Portc.2 , Db5 = Portc.3 , Db6 = Portc.4 , Db7 = Portc.5 , E = Portc.1 , Rs = Portc.0
Config Lcd = 16 * 1
Config I2cdelay = 1                     'Use the fastest I2C clock
'
'****************************************************************************
'* Define Ports & Data Direction Registers  (In = 0, Out = 1)               *
'****************************************************************************
' Port B
Portb = &B0000_0000                     'Set All Port Pins Low
Ddrb = &B1111_0111                      'Set encoder pins as inputs (tri-state)
'
' Port C
Portc = &B0000_0000                     'Set All Port Pins Low
Ddrc = &B1011_1111                      'Set pins as outputs
'
' Port D
Portd = &B0000_0000                     'Set All Port Pins Low
Ddrd = &B1111_0000                      'Set pins as inputs and outputs
Reset Ucr.3                             'Disable UART
Reset Ucr.4                             'Disable UART
'
'****************************************************************************
'* Define I2C pins                                                          *
'****************************************************************************
Config Sda = Portb.0                    ' I2C Data.
Config Scl = Portb.1                    ' I2C Clock
'
'****************************************************************************
Const Adresw = &HA0                     'write address of 24C256
Const Adresr = &HA1                     'read address of 24C256

Const Ds1307w = &HD0                    'Address Of DS1307 Write
Const Ds1307r = &HD1                    'Address Of DS1307 Read
'
Const Ds1307_add_sec = &H00             'Address Of DS1307 Seconds
Const Ds1307_add_min = &H01             'Address Of DS1307 Min
Const Ds1307_add_hour = &H02            'Address Of DS1307 Hour
Const Ds1307_add_control = &H07         'Address Of DS1307 Control
'
Const Ds1307_add_strt_min = &H10        'Address Of DS1307 Start Min
Const Ds1307_add_strt_hr = &H11         'Address Of DS1307 Start Hour
Const Ds1307_add_stp_min = &H12         'Address Of DS1307 Stop Min
Const Ds1307_add_stp_hr = &H13          'Address Of DS1307 Stop Hour
'
Const Ds1307_add_stat = &H20            'Address Of DS1307 Status
'
Const Io_1 = &H72                       'Address of IO Card1 (A0     = 1)
Const Io_2 = &H74                       'Address of IO Card2 (A1     = 1)
Const Io_3 = &H76                       'Address of IO Card3 (A0,A1  = 1)
Const Io_4 = &H78                       'Address of IO Card4 (A2     = 1)
'****************************************************************************
'* Init Variables                                                           *
'****************************************************************************
Display_mode = 255
New_sw = 1
Led_data = &H00_00
Pat_end = 30                            'Duration of each pattern (seconds)
Seed1 = 130                             'Duration in usec
'****************************************************************************
'* Power On Reset                                                           *
'****************************************************************************
   Restore Charz1
      For Tmpw_a = 1 To 23
         Read Char_u(tmpw_a)
         Next Tmpw_a
'
   Restore Charz2
      For Tmpw_a = 1 To 23
         Read Char_d(tmpw_a)
         Next Tmpw_a
'
Gosub Clear_arrays                      'Turn off all lamps
Gosub Start_rtc                         'Reset RTC
'
Portb.6 = 1                             'Turn off LED Display
Portb.7 = 1
'
Initlcd                                 ' start LCD display
Wait 1                                  'Delay until LCD and uC clock running
Cursor Off
Cls
Locate 1 , 1                            'Display welcome screen on LCD
Lcd " N&V Lightshow  "
Wait 2
Locate 1 , 1
Lcd "Ver 1.2 (c) 2008"
Wait 2
Locate 1 , 1
Lcd "Coded by Bigglez"
Wait 2
'
Config Timer1 = Timer , Prescale = 8    'timer1 generates 64*120Hz clk
Load Timer1 , Seed1
'
Enable Int0
'
On Timer1 Timer1_isr
On Int0 Int_isr
'
Enable Interrupts                       'Enable interrupts
'****************************************************************************
'* Main Program                                                             *
'****************************************************************************
Main:
'Set up for show
If Show = 0 Then
   Up_dwn = 0
   Pat_dly = 0
   Pat_num = 1
   Swing = 0
   Tempi = &H01
   Gosub Clear_arrays
   End If
'
Do
'Advance Pattern
If Show = 1 And Int_1sec1 = 1 Then
   Int_1sec1 = 0
   Incr Pat_dly
   If Pat_dly = Pat_end Then
     Incr Pat_num
     If Pat_num = 12 Then Pat_num = 1
     Up_dwn = 0
     Pat_dly = 0
      'Swing = 0
     Tempi = &H01
     End If
   End If
'
'     ############################################
'     # Pattern 1                                #
'     # Fast Marquee L to R Tempi = seed         #
'     ############################################
If Int_esec1 = 1 And Show = 1 And Pat_num = 1 Then
   Int_esec1 = 0
   For Tempm = 0 To 7
      If Tempi.tempm = 1 Then On_data(tempm + 1) = &H08
      If Tempi.tempm = 0 Then On_data(tempm + 1) = &H00
      Next Tempm
   Rotate Tempi , Right

   End If
'
'     ############################################
'     # Pattern 2                                #
'     # Fill and Flush R to L                    #
'     ############################################
If Int_esec1 = 1 And Show = 1 And Pat_num = 2 Then
   Int_esec1 = 0
   '
   For Tempm = 0 To 7
      If Tempi.tempm = 1 Then On_data(tempm + 1) = &H08
      If Tempi.tempm = 0 Then On_data(tempm + 1) = &H00
      Next Tempm
   If Swing = 0 Then
      Rotate Tempi , Left
      Tempi.0 = 1
      End If
'
   If Swing = 1 Then
      Rotate Tempi , Right
      Tempi.7 = 0
      End If
'
   If Tempi = &HFF Or Tempi = &H00 Then Toggle Swing
   End If
'
'     ############################################
'     # Pattern 3                                #
'     # Wave L to R                              #
'     ############################################
If Int_esec1 = 1 And Show = 1 And Pat_num = 3 Then
   Int_esec1 = 0
   '
If Pat_dly = 0 Then
   For Tempn = 0 To 7
      On_data(tempn + 1) = Tempn
      Next Tempn
      Ramp = &H00
   End If
'
   If Ramp.0 = 1 Then
      Decr On_data(1)
      Else
      Incr On_data(1)
      End If
'
   If On_data(1) = 0 Or On_data(1) = 8 Then Toggle Ramp.0
'
   If Ramp.1 = 1 Then
      Decr On_data(2)
      Else
      Incr On_data(2)
      End If
'
   If On_data(2) = 0 Or On_data(2) = 8 Then Toggle Ramp.1
'
   If Ramp.2 = 1 Then
      Decr On_data(3)
      Else
      Incr On_data(3)
      End If
'
   If On_data(3) = 0 Or On_data(3) = 8 Then Toggle Ramp.2
'
   If Ramp.3 = 1 Then
      Decr On_data(4)
      Else
      Incr On_data(4)
      End If
'
   If On_data(4) = 0 Or On_data(4) = 8 Then Toggle Ramp.3
'
   If Ramp.4 = 1 Then
      Decr On_data(5)
      Else
      Incr On_data(5)
      End If
'
   If On_data(5) = 0 Or On_data(5) = 8 Then Toggle Ramp.4
'
   If Ramp.5 = 1 Then
      Decr On_data(6)
      Else
      Incr On_data(6)
      End If
'
   If On_data(6) = 0 Or On_data(6) = 8 Then Toggle Ramp.5
'
   If Ramp.6 = 1 Then
      Decr On_data(7)
      Else
      Incr On_data(7)
      End If
'
   If On_data(7) = 0 Or On_data(7) = 8 Then Toggle Ramp.6
'
   If Ramp.7 = 1 Then
      Decr On_data(8)
      Else
      Incr On_data(8)
      End If
'
   If On_data(8) = 0 Or On_data(8) = 8 Then Toggle Ramp.7
   End If
'
'     ############################################
'     # Pattern 4                                #
'     # Fill and Flush L to R                    #
'     ############################################
If Int_esec1 = 1 And Show = 1 And Pat_num = 4 Then
   Int_esec1 = 0
   '
   For Tempm = 0 To 7
      If Tempi.tempm = 1 Then On_data(tempm + 1) = &H00
      If Tempi.tempm = 0 Then On_data(tempm + 1) = &H08
      Next Tempm
   If Swing = 0 Then
      Rotate Tempi , Left
      Tempi.0 = 1
      End If

   If Swing = 1 Then
      Rotate Tempi , Right
      Tempi.7 = 0
      End If

   If Tempi = &HFF Or Tempi = &H00 Then Toggle Swing
   End If
'
'     ############################################
'     # Pattern 5                                #
'     # Fast Marquee R to L Tempi = seed         #
'     ############################################
If Int_esec1 = 1 And Show = 1 And Pat_num = 5 Then
   Int_esec1 = 0
   '
   For Tempm = 0 To 7
      If Tempi.tempm = 1 Then On_data(tempm + 1) = &H08
      If Tempi.tempm = 0 Then On_data(tempm + 1) = &H00
      Next Tempm
   Rotate Tempi , Left
   End If
'
'     ############################################
'     # Pattern 6                                #
'     # Pendulum                                 #
'     ############################################
If Int_esec1 = 1 And Show = 1 And Pat_num = 6 Then
   Int_esec1 = 0
   '
   For Tempm = 0 To 7
      If Tempi.tempm = 1 Then On_data(tempm + 1) = &H08
      If Tempi.tempm = 0 Then On_data(tempm + 1) = &H00
      Next Tempm
   If Swing = 0 Then Rotate Tempi , Left
   If Swing = 1 Then Rotate Tempi , Right
   If Tempi.7 = 1 Then Swing = 1
   If Tempi.0 = 1 Then Swing = 0
   End If
'
'     ############################################
'     # Pattern 7                                #
'     # Fast ramp up and down all bulbs          #
'     ############################################
If Int_esec1 = 1 And Show = 1 And Pat_num = 7 Then
   Int_esec1 = 0
   '
   If On_data(1) = 0 Then
      For Tempm = 1 To 8
         On_data(tempm) = 0
         Next Tempm
         End If
'
   If Up_dwn = 0 Then
      For Tempm = 0 To 7
         Incr On_data(tempm + 1)
         If On_data(tempm + 1) > 8 Then
            On_data(tempm + 1) = &H08
            Done.tempm = 1
            End If
         Next Tempm
         If Done = &HFF Then Up_dwn = 1
         End If
'
   If Up_dwn = 1 Then
      For Tempm = 0 To 7
         Decr On_data(tempm + 1)
         If On_data(tempm + 1) < 1 Then
            On_data(tempm + 1) = &H00
            Done.tempm = 0
            End If
         Next Tempm
         If Done = 0 Then Up_dwn = 0
         End If
   End If
'
'     ############################################
'     # Pattern 8                                #
'     # Fill and Flush R to L (Random length)    #
'     ############################################
If Int_hsec1 = 1 And Show = 1 And Pat_num = 8 Then
   Int_hsec1 = 0
   '
   For Tempm = 1 To 8
      On_data(tempm) = &H00
      Next Tempm
'
   If Tempi = Tempn Or Tempi = 9 Then
      Tempi = &H00
      Tempn = Rnd(9)
      End If

   If Tempi > 0 Then
      For Tempm = 1 To Tempi
         On_data(tempm) = &H08
         Next Tempm
      End If
'
   Incr Tempi
   End If
'
'     ############################################
'     # Pattern 9                                #
'     # Fill and Flush L to R (Random length)    #
'     ############################################
If Int_hsec1 = 1 And Show = 1 And Pat_num = 9 Then
   Int_hsec1 = 0
   '
   For Tempm = 1 To 8
      On_data(tempm) = &H00
      Next Tempm
'
   If Tempi = Tempn Or Tempi = 0 Then
      Tempi = 9
      Tempn = Rnd(9)
      End If

   If Tempi < 9 Then
      For Tempm = Tempi To 8
         On_data(tempm) = &H08
         Next Tempm
      End If
'
   Decr Tempi
   End If
'
'     ############################################
'     # Pattern 10                               #
'     # Random single fast blinker (full bright) #
'     ############################################
If Int_esec1 = 1 And Show = 1 And Pat_num = 10 Then
   Int_esec1 = 0
   For Tempm = 1 To 8
      On_data(tempm) = &H00
      Next Tempm
   Tempn = Rnd(256)
   Tempn = Tempn And &H07
   Incr Tempn
   On_data(tempn) = &H08
   End If
'
'     ############################################
'     # Pattern 11                               #
'     # Random hits that fade out                #
'     ############################################
If Int_esec1 = 1 And Show = 1 And Pat_num = 11 Then
   Int_esec1 = 0
   For Tempn = 0 To 7
      If On_data(tempn + 1) > 0 Then Decr On_data(tempn + 1)
      If On_data(tempn + 1) = 0 Then Done.tempn = 1
      Next Tempn
   End If

If Done = &HFF And Int_1sec2 = 1 And Pat_num = 11 Then
   Tempm = Rnd(256)
   For Tempn = 0 To 7
   If Tempm.tempn = 1 Then On_data(tempn + 1) = &H08
   Next Tempn
   Done = 0
   Int_1sec2 = 0
   End If
'

'****************************************************************************
' RTC read and write can only be done when the I2C bus is not required for
' triac control during the AC half-cycle. RTC_en = 1 when the I2C bus is
' available.
If Rtc_en = 1 Then                      'Read and write to I2C during AC-cross
   Rtc_en = 0
   '
   If R_eeprom = 1 And W_eeprom = 0 Then
      Gosub Read_eeprom                 'Read one byte of the EEPROM
      R_eeprom = 0                      'Clear flags
      'Rtc_en = 0
      End If
   '
   If W_eeprom = 1 And R_eeprom = 0 Then
      Gosub Write_eeprom                'Write one byte of the EEPROM
      W_eeprom = 0                      'Clear flags
      'Rtc_en = 0
      End If
'****************************************************************************
   If R_rtc = 1 And W_rtc = 0 Then
      Gosub Read_rtc
      R_rtc = 0
      'Rtc_en = 0
      End If
   '
   If R_stat = 1 And W_stat = 0 Then
      Gosub Read_stat
      R_stat = 0
      'Rtc_en = 0
      End If
   '
   If R_stop = 1 And W_stop = 0 Then
      Gosub Read_stop
      R_stop = 0
      'Rtc_en = 0
      End If
   '
   If R_strt = 1 And W_strt = 0 Then
      Gosub Read_strt
      R_strt = 0
      'Rtc_en = 0
      End If
   '
'****************************************************************************
   If W_rtc = 1 And R_rtc = 0 Then
      Gosub Write_rtc
      W_rtc = 0
      'Rtc_en = 0
      End If
   '
   If W_stat = 1 And R_stat = 0 Then
      Gosub Write_stat
      W_stat = 0
      'Rtc_en = 0
      End If
   '
   If W_stop = 1 And R_stop = 0 Then
      Gosub Write_stop
      R_stop = 0
      'Rtc_en = 0
      End If
'
   If W_strt = 1 And R_strt = 0 Then
      Gosub Write_strt
      W_strt = 0
      End If
'
End If
'****************************************************************************
If Switch_data = 0 And Int_3sec = 1 Then
   Int_3sec = 0
   Incr Display_mode
   If Display_mode = 5 Then Display_mode = 1       'Display RTC time
   End If
'Display_mode = 4
'     #######################################
'     # Display_mode = 0 Display Status     #
'     # Display_mode = 1 Display RTC        #
'     # Display_mode = 2 Display Start      #
'     # Display_mode = 3 Display Stop       #
'     # Display_mode = 4 Display Show       #
'     #######################################
'
'****************************************************************************
'Display Status                        'Only during boot
'(If Int_qsec = 1 And Display_mode = 0 Then
'
   If Switch_data = 0 Then
      Enc_mode = 1
      R_stat = 1
      Ch_data.0 = Status.0
      Ch_data.1 = Status.1
      Led_inv = Status.4
      Mode_data = Status.5

      End If
'
   Led_data = &H12_15                   'Write "S-" to LED
   Lcd_msg = "CH:   D:   M:   "         'Write Status template to LCD
'
   Select Case Ch_data                  'Write data to LCD
      Case 0
      Mid(lcd_msg , 5 , 1) = "8"
      Led_data1 = &H16_08
      Case 1
      Mid(lcd_msg , 4 , 2) = "16"
      Led_data1 = &H01_06
      Case 2
      Mid(lcd_msg , 4 , 2) = "24"
      Led_data1 = &H02_04
      Case 3
      Mid(lcd_msg , 4 , 2) = "32"
      Led_data1 = &H03_02
      End Select
'
   Select Case Led_inv                  'Write data to LCD
      Case 0
      Mid(lcd_msg , 9 , 2) = "UP"
      Led_data2 = &H14_11
      Case 1
      Mid(lcd_msg , 9 , 2) = "DN"
      Led_data2 = &H0B_0F
      End Select
'
   Select Case Mode_data                'Write data to LCD
      Case 0
      Mid(lcd_msg , 14 , 2) = "EE"
      Led_data3 = &H0C_0C
      Case 1
      Mid(lcd_msg , 14 , 2) = "Fn"
      Led_data3 = &H0D_0F
      End Select
'
   If Switch_data = 1 Then              'Change Ch (Channel count)
      Enc_mode = 1
      Mid(lcd_msg , 3 , 1) = "<"
      Mid(lcd_msg , 6 , 1) = ">"
      Led_data = Led_data1
      End If
'
   If Switch_data = 2 Then
      Enc_mode = 2
      Mid(lcd_msg , 8 , 1) = "<"
      Mid(lcd_msg , 11 , 1) = ">"
      Led_data = Led_data2
      End If
'
   If Switch_data = 3 Then
      Enc_mode = 3
      Mid(lcd_msg , 13 , 1) = "<"
      Mid(lcd_msg , 16 , 1) = ">"
      Led_data = Led_data3
      End If
'
   If Switch_data = 4 Then
      Status.0 = Ch_data.0
      Status.1 = Ch_data.1
      Status.4 = Led_inv
      Status.5 = Mode_data
      W_stat = 1
      Switch_data = 0
      Display_mode = 0                  'Return to time display
      End If
'
   Int_qsec = 0
End If
')
'****************************************************************************
'Display RTC time
If Int_qsec = 1 And Display_mode = 1 Then
   Int_qsec = 0
   '
   If Switch_data = 0 Then
      R_rtc = 1
      Data_hr = Rtc_hr
      Data_min = Rtc_min
      Data_sec = Rtc_sec
      End If
'
   Led_data = &H0A_0E                   'Write "CL" to LED
   Lcd_msg = "Time:   :  :    "         'Write Time template to LCD
   Tmpb_b = Data_sec And &H0F
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 14 , 1) = Tmpb_b       'Write seconds data to LCD
   Tmpb_b = Data_sec And &HF0
   Shift Tmpb_b , Right , 4
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 13 , 1) = Tmpb_b
'
   Tmpb_b = Data_min And &H0F
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 11 , 1) = Tmpb_b       'Write mins data to LCD
   Tmpb_b = Data_min And &HF0
   Shift Tmpb_b , Right , 4
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 10 , 1) = Tmpb_b
'
   Tmpb_b = Data_hr And &H0F
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 8 , 1) = Tmpb_b        'Write mins data to LCD
   Tmpb_b = Data_hr And &HF0
   Shift Tmpb_b , Right , 4
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 7 , 1) = Tmpb_b
'
   If Switch_data = 1 Then
      Enc_mode = 4
      Pos_data = 6
      Led_data = Data_hr And &H00_F0
      Shift Led_data , Left , 4
      Tmpw_a = Data_hr And &H00_0F
      Led_data = Led_data Or Tmpw_a
      End If
'
   If Switch_data = 2 Then
      Enc_mode = 5
      Pos_data = 9
      Led_data = Data_min And &H00_F0
      Shift Led_data , Left , 4
      Tmpw_a = Data_min And &H00_0F
      Led_data = Led_data Or Tmpw_a
      End If
'
   If Switch_data = 3 Then
      Pos_data = 6
      Rtc_hr = Data_hr
      Rtc_min = Data_min
      Rtc_sec = Data_sec
      R_rtc = 0
      W_rtc = 1
      Switch_data = 0
      Display_mode = 0                  'Return to time display
      End If
'
   If Switch_data <> 0 Then
      Mid(lcd_msg , Pos_data , 1) = "<"
      Tmpb_b = Pos_data + 3
      Mid(lcd_msg , Tmpb_b , 1) = ">"
      End If
'
   End If
'
'Display Start time
If Int_qsec = 1 And Display_mode = 2 Then
   Int_qsec = 0
   '
   If Switch_data = 0 Then
      R_strt = 1
      Data_min = Strt_min
      Data_hr = Strt_hr
      End If
   '
   Led_data = &H12_13                   'Write "St" to LED
   Lcd_msg = "Start:  :       "         'Write Start template to LCD
   '
   Tmpb_b = Data_min And &H0F
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 11 , 1) = Tmpb_b       'Write mins data to LCD
   Tmpb_b = Data_min And &HF0
   Shift Tmpb_b , Right , 4
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 10 , 1) = Tmpb_b
   '
   Tmpb_b = Data_hr And &H0F
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 8 , 1) = Tmpb_b        'Write mins data to LCD
   Tmpb_b = Data_hr And &HF0
   Shift Tmpb_b , Right , 4
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 7 , 1) = Tmpb_b
   '
   If Switch_data <> 0 Then
    Mid(lcd_msg , Pos_data , 1) = "<"
    Tmpb_b = Pos_data + 3
    Mid(lcd_msg , Tmpb_b , 1) = ">"
    End If
   '
   If Switch_data = 1 Then
      Enc_mode = 4
      Pos_data = 6
      Led_data = Data_hr And &H00_F0
      Shift Led_data , Left , 4
      Tmpw_a = Data_hr And &H00_0F
      Led_data = Led_data Or Tmpw_a
      End If
'
   If Switch_data = 2 Then
      Enc_mode = 5
      Pos_data = 9
      Led_data = Data_min And &H00_F0
      Shift Led_data , Left , 4
      Tmpw_a = Data_min And &H00_0F
      Led_data = Led_data Or Tmpw_a
      End If

   If Switch_data = 3 Then
      Enc_mode = 1
      Pos_data = 6
      Strt_min = Data_min
      Strt_hr = Data_hr
      R_strt = 0
      W_strt = 1
      Switch_data = 0
      Display_mode = 1
      Show = 0
      End If
   '

   End If
'
'Display Stop time
If Int_qsec = 1 And Display_mode = 3 Then
   Int_qsec = 0
   '
   If Switch_data = 0 Then
      R_stop = 1
      Data_min = Stop_min               'Copy from RTC EEPROM to encoder & LED
      Data_hr = Stop_hr
      End If
'
   Led_data = &H12_11                   'Write "Sp" to LED
   Lcd_msg = "Stop:   :       "         'Write Stop template to LCD
   Tmpb_b = Data_min And &H0F
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 11 , 1) = Tmpb_b       'Write mins data to LCD
   Tmpb_b = Data_min And &HF0
   Shift Tmpb_b , Right , 4
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 10 , 1) = Tmpb_b
'
   Tmpb_b = Data_hr And &H0F
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 8 , 1) = Tmpb_b        'Write hrs data to LCD
   Tmpb_b = Data_hr And &HF0
   Shift Tmpb_b , Right , 4
   Tmpb_b = Tmpb_b + 48
   Mid(lcd_msg , 7 , 1) = Tmpb_b
'
   If Switch_data <> 0 Then
      Mid(lcd_msg , Pos_data , 1) = "<"
      Tmpb_b = Pos_data + 3
      Mid(lcd_msg , Tmpb_b , 1) = ">"
      End If
'
   If Switch_data = 1 Then
      Enc_mode = 4
      Pos_data = 6
      Led_data = Data_hr And &H00_F0
      Shift Led_data , Left , 4
      Tmpw_a = Data_hr And &H00_0F
      Led_data = Led_data Or Tmpw_a
      End If
'
   If Switch_data = 2 Then
      Enc_mode = 5
      Pos_data = 9
      Led_data = Data_min And &H00_F0
      Shift Led_data , Left , 4
      Tmpw_a = Data_min And &H00_0F
      Led_data = Led_data Or Tmpw_a
      End If
'
   If Switch_data = 3 Then
      Enc_mode = 1
      Pos_data = 6
      Stop_min = Data_min
      Stop_hr = Data_hr
      R_stop = 0
      W_stop = 1
      Switch_data = 0
      Display_mode = 1
      Show = 0
      End If
   End If
'
'Test for show run or idle
If Int_qsec = 1 And Display_mode = 4 Then
      Int_qsec = 0
      '
      St_mins = Makedec(strt_hr)        'Get start time and convert to minutes
      St_mins = St_mins * 60
      St_mins = St_mins + Makedec(strt_min)
      '
      Sp_mins = Makedec(stop_hr)        'Get stop time and convert to minutes
      Sp_mins = Sp_mins * 60
      Sp_mins = Sp_mins + Makedec(stop_min)
      '
      If St_mins > Sp_mins Then Ro_flg = 1       'Show runs past midnight
      If St_mins < Sp_mins Then Ro_flg = 0       'Show ends before midnight
      If Now_mins = 0 And Show = 1 Then Ro_flg = 0       'On at Midnight, Reset flag

      Now_mins = Makedec(rtc_hr)        'Get current time and convert to minutes
      Now_mins = Now_mins * 60
      Now_mins = Now_mins + Makedec(rtc_min)
'
   If Switch_data = 0 Then
      If St_mins <= Now_mins Then       'Start the show
         Show = 1
         End If
'
      If Now_mins >= Sp_mins And Ro_flg = 0 Then       'Stop the show
         Show = 0
         Gosub Clear_arrays
         End If
      End If
'
   If Switch_data = 0 Or Switch_data = 1 Then
      If Show = 1 Then Led_data = &H10_0F       'Write "On" to LED
'
      If Show = 0 Then
        Led_data = &H10_0D              'Write "Of" to LED
        Gosub Clear_arrays
        End If
      End If
'
   Lcd_msg = "Show:           "         'Write Show template to LCD
   If Show = 0 Then Mid(lcd_msg , 7 , 4) = "Idle"
   If Show = 1 Then Mid(lcd_msg , 7 , 7) = "Running"
   If Ro_flg = 0 Then Mid(lcd_msg , 14 , 2) = "  "
   If Ro_flg = 1 Then Mid(lcd_msg , 14 , 2) = "RO"
'
   If Switch_data = 1 Then
      Pos_data = 6
      Show = Not Enc_dir
      End If
'
   If Switch_data = 2 Then
      Switch_data = 0
      Display_mode = 3
      Show = 0
      End If
'
   End If
'
If Read_sw = 1 Then
   Read_sw = 0
   If Switch <> New_sw Then Toggle New_sw       'debounce switch
   If New_sw = 0 And Old_sw = 1 Then
     Incr Switch_data
     If Switch_data = 7 Then Switch_data = 0
     End If
  Old_sw = New_sw
'     #######################################
'     # Display_mode = 1 Display RTC        #
'     # Encoder Modes:                      #
'     # 0 Encoder changes data (0 to 99)    #
'     # 1 Encoder changes CH (8,16,24,or 32)#
'     # 2 Encoder changes D (Up or Dn)      #
'     # 3 Encoder changes M (EE or Pt)      #
'     # 4 Encoder changes hours (0 to 23)   #
'     # 5 Encoder changes minutes (0 to 59) #
'     #######################################
'
  ' Check Encoder for new data
     Enc_all = Enc_all And &H03         'recover last setting
     Enc_last = Enc_all
     Shift Enc_all , Left , 2
     Enc_now.0 = Enca
     Enc_now.1 = Encb
     Enc_all = Enc_all Or Enc_now
     '
     Select Case Enc_all
     Case 1
     Enc_dir = 0
     Case 2
     Enc_dir = 1
     Case 4
     Enc_dir = 1
     Case 7
     Enc_dir = 0
     Case 8
     Enc_dir = 0
     Case 11
     Enc_dir = 1
     Case 13
     Enc_dir = 1
     Case 14
     Enc_dir = 0
     End Select
'
     If Enc_mode = 1 Then
        If Enc_last <> Enc_now And Enc_dir = 1 Then Decr Ch_data
        If Ch_data = 255 Then Ch_data = 0
        If Enc_last <> Enc_now And Enc_dir = 0 Then Incr Ch_data
        If Ch_data = 4 Then Ch_data = 3
        End If
'
     If Enc_mode = 2 Then
        If Enc_last <> Enc_now And Enc_dir = 1 Then Toggle Led_inv
        If Enc_last <> Enc_now And Enc_dir = 0 Then Toggle Led_inv
        End If
'
'
     If Enc_mode = 3 Then
        If Enc_last <> Enc_now And Enc_dir = 1 Then Toggle Mode_data
        If Enc_last <> Enc_now And Enc_dir = 0 Then Toggle Mode_data
        End If
  '
     If Enc_mode = 4 Then
        Data_hr_d = Makedec(data_hr)
        If Enc_last <> Enc_now And Enc_dir = 1 Then Decr Data_hr_d
        If Data_hr_d = 255 Then
            Data_hr_d = 23
            'RO_flg = 0                'Same day
            End If
        If Enc_last <> Enc_now And Enc_dir = 0 Then Incr Data_hr_d
        If Data_hr_d > 23 Then
            Data_hr_d = 0
            'RO_flg = 1                'Next day
            End If
        Data_hr = Makebcd(data_hr_d)
        End If
  '
     If Enc_mode = 5 Then
        Data_min_d = Makedec(data_min)
        If Enc_last <> Enc_now And Enc_dir = 1 Then Decr Data_min_d
        If Data_min_d > 60 Then Data_min_d = 59
        If Enc_last <> Enc_now And Enc_dir = 0 Then Incr Data_min_d
        If Data_min_d = 60 Then Data_min_d = 0
        Data_min = Makebcd(data_min_d)
        End If
'
   End If
'
If Lcd_en = 1 And Int_lcd = 1 Then      'Write message data to LCD module
   Int_lcd = 0
   Locate 1 , Cnt_lcd
   Lcd = Mid(lcd_msg , Cnt_lcd , 1)
   Incr Cnt_lcd                         ' Advance LCD cursor counter
   If Cnt_lcd = 17 Then Cnt_lcd = 1
   End If
'
Loop                                    'Repeat forever
'
'****************************************************************************
'* ISR Interrupt Service Routines                                           *
'****************************************************************************
Int_isr:
' Handles 120Hz activity. Detects first crossing of each AC Half-cycle
' This interrupt is level not edge triggered so it must be locked out
' immediately after the first hit. The Int0 is enabled again near the
' expected next crossing (cnt1 = 48). If AC crossing sync is lost the
' watchdog timer (timer0) will generate a fake reset until sync is
' regained. int_isr execution = 500us
Disable Int0
'
Load Timer1 , Seed1                     'Restart AC event counter
Enable Timer1                           'Start AC event counter
Cnt1 = 0
Cnt2 = 0
'
Toggle Led_en
Portb.2 = 1                             'Trigger 'scope
I2csend Io_1 , &HFF                     'Turn off all Io_1 channels
I2csend Io_2 , &HFF                     'Turn off all Io_2 channels
I2csend Io_3 , &HFF                     'Turn off all Io_3 channels
I2csend Io_4 , &HFF                     'Turn off all IO_4 channels
Portb.2 = 0
'
If Led_en = 0 Then
   Lcd_en = 1
   End If
'
If Led_en = 1 Then
   Rtc_en = 1
   End If

For Tmpb_b = 1 To 16                    'Reset lamp data to &HFF (lamps off)
   Lamp_dat1(tmpb_b) = &HFF
   Next Tmpb_b
'
For Tmpb_c = 0 To 7
'
   Tmpb_a = 9 - On_data(tmpb_c + 1)
   For Tmpb_b = Tmpb_a To 8
      Lamp_dat1(tmpb_b).tmpb_c = 0
      Next Tmpb_b
'
   Tmpb_a = On_data(tmpb_c + 1)
   For Tmpb_b = Tmpb_a To 8
      Lamp_dat2(tmpb_b).tmpb_c = 1
      Next Tmpb_b
'
   Next Tmpb_c
'
Read_sw = 1                             'Read the switch on each AC crossing
Int_lcd = 1
'
Incr Esec_cnt
If Esec_cnt = 15 Then
   Int_esec1 = 1

   Int_esec = 1                         '1/8 Second interrupt
   Esec_cnt = 1
   End If
'
Incr Qsec_cnt
If Qsec_cnt = 30 Then
   Int_qsec1 = 1
   Int_qsec = 1                         '1/4 Second interrupt
   Qsec_cnt = 1

   End If
'
Incr Hsec_cnt
If Hsec_cnt = 60 Then
   Int_hsec1 = 1
   Int_hsec = 1                         '1/2 Second interrupt
   Hsec_cnt = 1
   End If
'
Incr 1sec_cnt
If 1sec_cnt = 120 Then
   Int_1sec1 = 1
   Int_1sec2 = 1
   Int_1sec = 1                         '1 Second interrupt
   1sec_cnt = 1
   Incr 3sec_cnt
   If 3sec_cnt = 3 Then
      Int_3sec = 1
      Int_3sec1 = 1                     '3 Second interrupt
      3sec_cnt = 0
      End If
   End If
'
Return
'****************************************************************************
Timer1_isr:
' This timer generates a string of 64 pulses (cnt1) during the AC
' Half-cycle. These are used to determine the active triac firing range
' and to inhibit any other I2C bus activity when the triacs have priority.
Load Timer1 , Seed1                     'reset timer1

' Prepare the lamp data to turn on selected channels at the right time-slot
' for correct brightness of each channel.
Incr Cnt1
Incr Cnt2

If Cnt2 < 6 Then
   Cnt1 = 0
   End If
'
If Cnt2 > 6 Then
   Cnt2 = 7
   End If
'
Select Case Cnt1

   Case 1
   Lcd_en = 0
   Rtc_en = 0
   Portb.2 = 0
'
   Case 2
     Portb.6 = 1                        'Turn off both LED digits
     Portb.7 = 1
'
   Case 3
   If Led_en = 1 Then
      Tmpw_a = Led_data
      Tmpw_a = Tmpw_a And &H00_FF
      If Led_inv = 0 Then Tmpw_a = Char_u(tmpw_a + 1)
      If Led_inv = 1 Then Tmpw_a = Char_d(tmpw_a + 1)
      Portd = Tmpw_a And &H00_F0
      Shift Tmpw_a , Right , 8
      Portc = Tmpw_a And &H00_7C
      If Led_inv = 0 Then Portb = Portb And &H7F       'Turn on left LED
      If Led_inv = 1 Then Portb = Portb And &HBF       'Turn on right LED
      Else
      End If
'
   Case 4
   I2csend Io_1 , Lamp_dat1(1)
'
   Case 8
   I2csend Io_1 , Lamp_dat1(2)
'
   Case 12
   I2csend Io_1 , Lamp_dat1(3)
'
   Case 16
   I2csend Io_1 , Lamp_dat1(4)
'
   Case 20
   Portb.6 = 1                          'Turn off both LED digits
   Portb.7 = 1
'
   Case 21
   If Led_en = 1 Then                   'Display LSD LED data
      Tmpw_a = Led_data
      Tmpw_a = Tmpw_a And &HFF_00
      Shift Tmpw_a , Right , 8
      If Led_inv = 0 Then Tmpw_a = Char_u(tmpw_a + 1)
      If Led_inv = 1 Then Tmpw_a = Char_d(tmpw_a + 1)
      Portd = Tmpw_a And &H00_F0
      Shift Tmpw_a , Right , 8
      Portc = Tmpw_a And &H00_7C
      If Led_inv = 0 Then Portb = Portb And &HBF       'Turn on right LED
      If Led_inv = 1 Then Portb = Portb And &H7F       'Turn on left LED
      End If
'
   Case 22
   I2csend Io_1 , Lamp_dat1(5)
'
   Case 26
   I2csend Io_1 , Lamp_dat1(6)
'
   Case 30
   I2csend Io_1 , Lamp_dat1(7)
'
   Case 34
       Portb.6 = 1                      'Turn off both LED digits
       Portb.7 = 1
'
   Case 35
   Enable Int0
'
   Case 36
   I2csend Io_1 , Lamp_dat1(8)
'
   Case 40
   Disable Timer1
   Cnt1 = 0
'
   End Select
'
Return
'****************************************************************************
'* Subroutines                                                              *
'****************************************************************************
Clear_arrays:                           'Set Up Data In On_data Array
For Tmpb_b = 1 To 8                     'Fill 32 bytes in on_data array
   Lamp_dat1(tmpb_b) = &HFF
   Next Tmpb_b
'
For Tmpb_b = 1 To 8
   On_data(tmpb_b) = &H00
   Next Tmpb_b
'
Return
'****************************************************************************
Read_eeprom:                            'Read from EEPROM
I2cstart                                'generate start
I2cwbyte Adresw                         'send mem address
I2cwbyte Adresh                         'high address
I2cwbyte Adresl                         'low address
I2cstart                                'generate repeated start
I2cwbyte &HA1                           'Set EEPROM to read
I2crbyte Ee_dat1 , Nack                 'receive value, last byte
I2cstop                                 'generate stop
Ee_dat2 = Ee_dat1 And &HF0              'unpack byte to two nibbles
Shift Ee_dat2 , Right , 4
Ee_dat1 = Ee_dat1 And &H0F
Return
'****************************************************************************
Read_rtc:                               ' Read Data over I2C
I2cstart                                ' RTC device address and data address
I2cwbyte Ds1307w                        ' Slave adsress
I2cwbyte Ds1307_add_sec                 ' Address of seconds register
I2cstop                                 ' Generate Stop
'
I2cstart                                ' Repeated start
I2cwbyte Ds1307r                        ' Slave address (read)
I2crbyte Rtc_sec , Ack                  ' Generate ACK
I2crbyte Rtc_min , Ack
I2crbyte Rtc_hr , Nack                  ' Generate NACK
I2cstop
'
Return
'****************************************************************************
Read_stat:
I2cstart                                'RTC device address and data address
I2cwbyte Ds1307w                        'Slave adsress
I2cwbyte Ds1307_add_stat                'Address of data register
I2cstop                                 'Generate Stop
'
I2cstart                                'Repeated start
I2cwbyte Ds1307r                        'Slave address (read)
I2crbyte Status , Nack                  'Generate NACK
I2cstop                                 'Generate Stop
'
Return
'****************************************************************************
Read_stop:
I2cstart                                'RTC device address and data address
I2cwbyte Ds1307w                        'Slave adsress
I2cwbyte Ds1307_add_stp_min             'Address of data register
I2cstop                                 'Generate Stop
'
I2cstart                                'Repeated start
I2cwbyte Ds1307r                        'Slave address (read)
I2crbyte Stop_min , Ack
I2crbyte Stop_hr , Nack                 'Generate NACK
I2cstop                                 'Generate Stop
'
Return
'****************************************************************************
Read_strt:
I2cstart                                'RTC device address and data address
I2cwbyte Ds1307w                        'Slave adsress
I2cwbyte Ds1307_add_strt_min            'Address of data register
I2cstop                                 'Generate Stop
'
I2cstart                                'Repeated start
I2cwbyte Ds1307r                        'Slave address (read)
I2crbyte Strt_min , Ack
I2crbyte Strt_hr , Nack                 'Generate NACK
I2cstop                                 'Generate Stop
'
Return
'****************************************************************************
Start_rtc:                              'Restart RTC
Tempx = &H00                            'Oscillator Enable (Bit7 = 0)
I2cstart
I2cwbyte Ds1307w
I2cwbyte Ds1307_add_sec                 'Write zero seconds
I2cwbyte Tempx
I2cstop
'
Tempx = &B1001_0000                     'Enable SQW Output (Bit4 = 1)
I2cstart                                'Select 1Hz (Bit1, Bit0 = 0)
I2cwbyte Ds1307w
I2cwbyte Ds1307_add_control             'Write Control byte
I2cwbyte Tempx
I2cstop
'
Return
'****************************************************************************
Write_eeprom:                           'Write to EEPROM
Tmp_ee1 = Ee_dat1 And &H0F              'pack two nibbles into byte
Tmp_ee2 = Ee_dat2 And &H0F
Shift Tmp_ee2 , Left , 4
Tmp_ee1 = Tmp_ee1 Or Tmp_ee2
I2cstart                                'generate start
I2cwbyte Adresw                         'send mem address
I2cwbyte Adresh                         'send mem low address
I2cwbyte Adresl                         'send address of EEPROM
I2cwbyte Tmp_ee1                        'send a value
I2cstop                                 'generate stop
Waitms 5                                'wait 5mS (write cycle time)
Return
'****************************************************************************
Write_rtc:                              'Send time data to RTC
Tempx = Rtc_sec And &B0111_1111         'Oscillator Enable (Bit7 = 0)
I2cstart
I2cwbyte Ds1307w
I2cwbyte Ds1307_add_sec                 'Write seconds
I2cwbyte Tempx
I2cstop
'
Tempx = Rtc_min And &B0111_1111         'Set Minutes
I2cstart
I2cwbyte Ds1307w
I2cwbyte Ds1307_add_min                 'Write minutes
I2cwbyte Tempx
I2cstop
'
Tempx = Rtc_hr And &B0011_1111          'Set 24 Hour Mode (Bit6 = 0)
I2cstart
I2cwbyte Ds1307w
I2cwbyte Ds1307_add_hour                'Write hours
I2cwbyte Tempx
I2cstop
'
Tempx = &B1001_0000                     'Enable SQW Output (Bit4 = 1)
I2cstart                                'Select 1Hz (Bit1, Bit0 = 0)
I2cwbyte Ds1307w
I2cwbyte Ds1307_add_control             'Write Control byte
I2cwbyte Tempx
I2cstop
'
Return
'****************************************************************************
Write_stat:                             'Send status to RTC memory
Tempx = Status
I2cstart
I2cwbyte Ds1307w
I2cwbyte Ds1307_add_stat                'Write Status
I2cwbyte Tempx
I2cstop
Waitms 5                                'wait 5mS (write cycle time)
'
Return
'****************************************************************************
Write_stop:                             'Send time data to RTC
Tempx = Stop_min
I2cstart
I2cwbyte Ds1307w
I2cwbyte Ds1307_add_stp_min             'Write Stop minutes
I2cwbyte Tempx
I2cstop
Waitms 5                                'wait 5mS (write cycle time)
'
Tempx = Stop_hr
I2cstart
I2cwbyte Ds1307w
I2cwbyte Ds1307_add_stp_hr              'Write Stop hours
I2cwbyte Tempx
I2cstop
Waitms 5                                'wait 5mS (write cycle time)
'
Return
'****************************************************************************
Write_strt:                             'Send time data to RTC
Tempx = Strt_min
I2cstart
I2cwbyte Ds1307w
I2cwbyte Ds1307_add_strt_min            'Write Start minutes
I2cwbyte Tempx
I2cstop
Waitms 5                                'wait 5mS (write cycle time)
'
Tempx = Strt_hr
I2cstart
I2cwbyte Ds1307w
I2cwbyte Ds1307_add_strt_hr             'Write Start hours
I2cwbyte Tempx
I2cstop
Waitms 5                                'wait 5mS (write cycle time)
'
Return
'
'****************************************************************************
End                                     'end program
'****************************************************************************
Charz1:                                 'Normal digit character map
Data &H0F , &HF0                        '0
Data &H9F , &HFC                        '1
Data &H2F , &HE8                        '2
Data &H0F , &HEC                        '3
Data &H9F , &HE4                        '4
Data &H4F , &HE4                        '5
Data &H4F , &HE0                        '6
Data &H1F , &HFC                        '7
Data &H0F , &HE0                        '8
Data &H0F , &HE4                        '9
'                                       'Normal Alpha character map
Data &H6F , &HF3                        'C (0a)
Data &H8F , &HEB                        'd (0b)
Data &H6F , &HE0                        'E (0c)
Data &H7F , &HE0                        'f (0d)
Data &HEF , &HF3                        'L (0e)
Data &HDF , &HEB                        'n (0f)
Data &H0F , &HF0                        'o (10)
Data &H3F , &HE3                        'p (11)
Data &H4F , &HE4                        'S (12)
Data &HEF , &HE3                        't (13)
Data &H8F , &HF3                        'U (14)
Data &HFF , &HEF                        '- (15)
Data &HFF , &HFF                        'blank  (16)
'
Charz2:                                 'Inverted character map
Data &H0F , &HF0                        '0
Data &HFF , &HF0                        '1
Data &H2F , &HE8                        '2
Data &H6F , &HE0                        '3
Data &HDF , &HE0                        '4
Data &H4F , &HE4                        '5
Data &H0F , &HE4                        '6
Data &HEF , &HF0                        '7
Data &H0F , &HE0                        '8
Data &H4F , &HE0                        '9
'                                       'Inverted Alpha character map
Data &H0F , &HFF                        'C (0a)
Data &H3F , &HE3                        'd (0b)
Data &H0F , &HEC                        'E (0c)
Data &H8F , &HEF                        'f (0d)
Data &H1F , &HFF                        'L (0e)
Data &HBF , &HE7                        'n (0f)
Data &H0F , &HF0                        'o (10)
Data &H8F , &HEB                        'p (11)
Data &H4F , &HE4                        'S (12)
Data &H1F , &HEF                        't (13)
Data &HDF , &HEB                        'u (14)
Data &HFF , &HEF                        '- (15)
Data &HFF , &HFF                        'blank (16)